PROVISION_SERVICE = 'PROVISION_SERVICE'
DELETE_SERVICE = 'DELETE_SERVICE'
NODE_METRICS = 'NODE_METRICS'
SERVICES_STATUS = 'SERVICES_STATUS'
DELETE_NODE = 'DELETE_NODE'
RESTART_SERVICE = 'RESTART_SERVICE'

SERVICE_UP = 'UP'
SERVICE_DOWN = 'DOWN'
SERVICE_RESTARTING = 'RESTARTING'

LIST_USB_DEVICES = 'lsusb -v'
FILTER_SMART_CARD_COMMANDS = ['grep -E "\<(ID|Chip/SmartCard)"', 'grep "Chip/SmartCard" -B 1']

UNINSTALL_SCRIPT_FILE = 'uninstall.sh'

DOCKER_COMPOSE_FILE = 'docker-compose.yaml'

EXCHANGE = "agent"
EXCHANGE_TYPE = "topic"
NOTIFY_CONTROL_CENTER = 'NOTIFY_CONTROL_CENTER'
